/*
** FF-Egg.c - Egg for the Food Fight Pak
**
** This Effect Pack uses the MULTIPLE DRAW Method to paint objects
**
** by Jamie Cristini
**
*/
 


#include <ddraw.h>
#include <stdio.h>
#include <windows.h>
#include <time.h>
#include <math.h>
#include "BlasterEffects.h"
#include "EffectType.h"
#include "FF-Main.h"

#define MAX_EGG_OBJECTS				5
#define MAX_EGG_DEFENSE_SPEED		3   // Lower number for a better defense
#define MIN_EGG_DEFENSE_SPEED		2	// Lower number for a better defense
#define MAX_EGG_DEFENSE_DISTANCE	100 // Determines the max pixels the defense can move in a single step

#define MOUSE_CURSOR	-1

#define TIME_BEFORE_EGG_HIT_BOUNCE	-10
#define TIME_FOR_EGG_TO_REACH		-9



void CalculateEggShots();
void CalculateEgg(void);
int UpdateEggEffect(DDRAW_DATA *dd_data, int nEffect, int nMsg, int nObjectCount);
int InitEffects(void);
int DrawEgg(DDRAW_DATA *dd_data, int nMsg);
void DrawEggBuffer(DDRAW_DATA *dd_data, int nPlayer);
void DrawEggScreen(DDRAW_DATA *dd_data, int nPlayer);
void EraseEggBuffer(DDRAW_DATA *dd_data, int nPlayer);
int CreateEggShot(DDRAW_DATA *dd_data);
void CheckEggButtons(DDRAW_DATA *dd_data, int nPlayer);
void MoveEggDefense(void);
void AssignEggPointers(int nPlayer);
void CalculateAnEgg(MULTIOBJECT *Obj);
int EggManualSave(DDRAW_DATA *dd_data, int nMsg, void* vDataStruct);
void CalculateEggDefense(void);
void CheckForEggCodes(void);

int i;

/*int InitEffects()
{
RegisterEffect("Rotten-Egg","Construction", DrawEgg, UpdateEggEffect, ".\\images\\Medieval\\Ce5Kitty.pcx", EggManualSave, NULL);		
return TRUE;
}*/

int UpdateEggEffect(DDRAW_DATA *dd_data, int nEffect, int nMsg, int nObjectCount)
{
int i;

switch(nMsg)
	{
	case EFFECT_STARTED:
		/* Egg */
		if(FindEffect("Rotten-Egg") == nEffect)
			{
			// Load Animated Cursor
			//LoadTexture( dd_data, ".\\Images\\co1gecr1.pcx", &img.AnimCur[0]);


			//Wave[3] = LoadWave(".\\Sounds\\noammo.wav");
			//Load Animated Cursor
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Cc4EggCr.pcx"  , &img.AnimCur[0]);

			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Egg.pcx", &img.WeaponEnd[0]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Crak1.pcx", &img.WeaponEnd[1]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Crak2.pcx", &img.WeaponEnd[2]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4EggTr2.pcx", &img.WeaponEnd[3]);

			// Load Defense image
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Cd4Tray.pcx", &img.Defenses[0]);

			Wave[0] = LoadWave(".\\Sounds\\Food Fight\\NoAmmo.wav");
			Wave[1] = LoadWave(".\\Sounds\\Food Fight\\Egg013.wav");
			Wave[2] = LoadWave(".\\Sounds\\Food Fight\\Egg01t.wav");


			InitObject(-3, 
				&dfP1Defense.fxObject, 
				&img.Defenses[0],
				nMouseX - (img.Defenses[0].width / 2),
				nMouseY - (img.Defenses[0].height/2));					

			
			// Load Egg Explosion textures


			// Load Defense Images
			//LoadTexture( dd_data, ".\\Images\\tennis1.pcx", &img.Defenses[0]);
			//LoadTexture( dd_data, ".\\Images\\tennis2.pcx", &img.Defenses[1]);
			//LoadTexture( dd_data, ".\\Images\\tennis3.pcx", &img.Defenses[2]);



			InitObject(MOUSE_CURSOR, 
					&fxPlayerOne, 
					&img.AnimCur[0],
					nMouseX - (img.AnimCur[0].width / 2),
					nMouseY - (img.AnimCur[0].height/2));


			/*InitObject(-1, 
						&dfDefense.fxDefense,
						&img.Defenses[0],
						nScreenWidth/2,
						nScreenHeight/2
						);*/

			for(i=0; i<MAX_EGG_OBJECTS; i++)
				{
				moP1Obj[i].byObjectAlive=FALSE;
				moP1Obj[i].byID = i;
				}

			break;
			}
		break;

	case EFFECT_ENDED:
		if(FindEffect("Rotten-Egg") == nEffect)
			{
			DeleteWaves();

			DeleteTexture(&img.AnimCur[0]);

			//for(i=0; i<12; i++)
			//	DeleteTexture(&img.Weapon[i]);

			for(i=0; i<4; i++)
				DeleteTexture(&img.WeaponEnd[i]);

			for(i=0; i<1; i++)
				DeleteTexture(&img.Defenses[i]);

			for(i=0; i<MAX_EGG_OBJECTS; i++)
				{
				if(moP1Obj[i].byObjectAlive==TRUE)
					{
					moP1Obj[i].byObjectAlive=FALSE;
					moP1Obj[i].byID = i;
					moP1Obj[i].pfxObject->nFlags=0;
					DeleteLastFXObject();
					}
				}


			break;
			}
	break;


	case URL_CHANGED:

	break;

	case DRAW_CREDITS:
		//DrawCredit("This is from teh egg pack");
	break;
	}

return TRUE;
}

int DrawEgg(DDRAW_DATA *dd_data, int nMsg)
{
dfP1Defense.byObjectAlive=nEffectsDefended;

switch(nMsg)
	{
	case DRAW_EFFECT:
		//AssignEggPointers(PLAYER_ONE);
		CalculateEgg();
		DrawEggBuffer(dd_data, PLAYER_ONE);
		DrawEggScreen(dd_data, PLAYER_ONE);
		EraseEggBuffer(dd_data, PLAYER_ONE);
	break;

	case CALCULATE_AND_SKIP_FRAME:
		//AssignEggPointers(PLAYER_ONE);
		CalculateEgg();
	break;
	}


CheckEggButtons(dd_data, PLAYER_ONE);
return TRUE;
}

void CalculateEgg()
{
fxPlayerOne.x = nMouseX - (img.AnimCur[fxPlayerOne.nFrame].width/2);
fxPlayerOne.y = nMouseY - (img.AnimCur[fxPlayerOne.nFrame].height/2);

CalculateEggShots();

if(dfP1Defense.byObjectAlive==TRUE)
   CalculateEggDefense();
}

void CalculateEggShots()
{
int i;

for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==TRUE)
		{
		switch(moP1Obj[i].pfxObject->nFlags)
			{
			case FALLING:
				moP1Obj[i].pfxObject->fAngle += 10;
				if(moP1Obj[i].pfxObject->fAngle>360)
					moP1Obj[i].pfxObject->fAngle = (float) 0.0;

				moP1Obj[i].pfxObject->fScaleY -= 5;
				moP1Obj[i].pfxObject->fScaleX -= 5;

				moP1Obj[i].pfxObject->x = (int) (moP1Obj[i].nStartX - moP1Obj[i].pfxObject->fScaleX/2);
				moP1Obj[i].pfxObject->y = (int) ((moP1Obj[i].nStartY + moP1Obj[i].d) - moP1Obj[i].pfxObject->fScaleY/2); 


				if(moP1Obj[i].t<-11)   // if TRUE then the Molotov has hit
					{
					PlayWave(Wave[1]);
									
					moP1Obj[i].pfxObject->nFlags = EXPLODING;
					moP1Obj[i].pfxObject->nFrame=1;
					moP1Obj[i].nDrawFlags = SCALE_IMAGE;  //fSCaleX = 71.0 when hits

					//moP1Obj[i].pfxObject->x = (int) (moP1Obj[i].nStartX - img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2 - 10);
					//moP1Obj[i].pfxObject->y = (int) ((moP1Obj[i].nStartY + moP1Obj[i].d) - img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].height/2 - 15);
					}
				else
					{
					moP1Obj[i].t += nFrameIncr;  // Increment Time

					moP1Obj[i].d = (float) (.5 * moP1Obj[i].g * (moP1Obj[i].t * moP1Obj[i].t));  // Find the new distance
					}
			break;

			case EXPLODING:
				moP1Obj[i].pfxObject->nFrameTime++;
				if(moP1Obj[i].pfxObject->nFrameTime>2)
					{
					moP1Obj[i].pfxObject->nFrameTime=0;
					moP1Obj[i].pfxObject->nFrame++;
					if(moP1Obj[i].pfxObject->nFrame>2)
						{
						moP1Obj[i].pfxObject->nFrame--;
						moP1Obj[i].pfxObject->nFlags = HIT;
						moP1Obj[i].nDrawFlags = SCALE_IMAGE;  //fSCaleX = 71.0 when hits
						}
					}

			break;


			case HIT:
			break;
			}
		}
	}

}

void DrawEggBuffer(DDRAW_DATA *dd_data, int nPlayer)
{
AssignEggPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	DrawObjectToBuffer(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame], 0);	

for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case BLOCKED:
			break;

			case HIT:
				DrawObjectToBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	
				DrawObjectToScreenCapture(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	
			break;

			default:
				DrawObjectToBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	

			}
		}
	}


// draw the weapon
DrawObjectToBuffer(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame], 0);	

}

void DrawEggScreen(DDRAW_DATA *dd_data, int nPlayer)
{
AssignEggPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	DrawBufferObjectToScreen(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	

if(dfP1Defense.fxObject.dwFlags==ERASE)
	EraseObjectOnScreen(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	


for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case BLOCKED:
				EraseObjectOnScreen(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
				moP1Obj[i].pfxObject->x = -9999;
				moP1Obj[i].pfxObject->y = -9999;
				moP1Obj[i].byObjectAlive=FALSE;
				moP1Obj[i].pfxObject->nFlags=0;
			break;

			default:
				DrawBufferObjectToScreen(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
			}

		}
	}


//draw the weapon
DrawBufferObjectToScreen(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame]);	


}

void EraseEggBuffer(DDRAW_DATA *dd_data, int nPlayer)
{
AssignEggPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	EraseObjectInBuffer(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	


for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case HIT:
				EraseObjectInBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
				pmoObj[i].byObjectAlive=FALSE;
			break;

			default:
				EraseObjectInBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
			}
		}
	}

//erase the weapon
EraseObjectInBuffer(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame]);	


}


void CheckEggButtons(DDRAW_DATA *dd_data, int nPlayer)
{
//AssignEggPointers(nPlayer);

/* UPDATE MOUSE BUTTONS */
if(LButtonPressed==TRUE)
	{
	LButtonPressed=FALSE;					
	
	if(CreateEggShot(dd_data)==TRUE)
		{
		nFrameIncr= -1;		// We want to increment time by a -1
		}
	}
 
if(RButtonPressed==TRUE)
	{
	RButtonPressed=FALSE;					
	dfP1Defense.byObjectAlive^=TRUE;
	}

CheckForEggCodes();
}


int CreateEggShot(DDRAW_DATA *dd_data)
{
int i;

for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==FALSE)
		break;

	if(i == (MAX_EGG_OBJECTS-1))
		return FALSE;
	}

if((moP1Obj[i].pfxObject = (FXOBJECT *) CreateFXObject()) == (FXOBJECT *) NULL)
	{
	PlayWave(Wave[0]);
	return FALSE;
	}

InitObject(FindEffect("Rotten-Egg"),
		   moP1Obj[i].pfxObject,
		   &img.Weapon[0],
		   nMouseX - (img.WeaponEnd[0].width/2),
		   nScreenHeight - (img.WeaponEnd[0].height/2));

moP1Obj[i].pfxObject->fScaleX = (float) 256;
moP1Obj[i].pfxObject->fScaleY = (float) 256;
moP1Obj[i].pfxObject->nFlags = FALLING;
moP1Obj[i].nDrawFlags = SCALE_IMAGE|ROTATE_IMAGE;


moP1Obj[i].nStartX = nMouseX; 
moP1Obj[i].nStartY = (int) (nMouseY-((nScreenHeight - nMouseY) * .33));  // .33 just looks good
moP1Obj[i].nDestX = nMouseX - (img.AnimCur[0].width/2);
moP1Obj[i].nDestY = nMouseY - (img.AnimCur[0].height/2);


moP1Obj[i].d = (float) (nScreenHeight - moP1Obj[i].nStartY); // the total distance
moP1Obj[i].t = (float) 24.0;				  // 24 frames until we hit
moP1Obj[i].g = (float) (moP1Obj[i].d / (.5 * (moP1Obj[i].t * moP1Obj[i].t))); // g = d / (.5 * t^2)

moP1Obj[i].byObjectAlive=TRUE;

return TRUE;
}

void MoveEggDefense(void)
{

}


void AssignEggPointers(int nPlayer)
{
switch(nPlayer)
	{
	case PLAYER_ONE:
		pfxPlayer = (FXOBJECT *) &fxPlayerOne;
		pfxWeapon = (FXOBJECT *) &fxP1Weapon;
		pmoObj =	(MULTIOBJECT *) &moP1Obj[0];
	break;

	case PLAYER_TWO:
	break;

	default:
		pfxWeapon = (FXOBJECT *) NULL;
		pmoObj = (MULTIOBJECT *) NULL;
	}
}



typedef struct _RottenEggData
{
int x;
int y;
DWORD dwFlags;

} EGGDATA;


int EggManualSave(DDRAW_DATA *dd_data, int nMsg, void* vDataStruct)
{
static TEXTURE tex;
static int nTextureLoaded=FALSE;
URLDATA *URLData;
int i=0;
int nType;
EGGDATA *cpData;
FXOBJECT *fx;
int nTotal=0;



switch(nMsg)
	{
	case MANUAL_DRAW_TO_BROWSER:
		{
		fx = (FXOBJECT *) vDataStruct;


		if(nTextureLoaded==FALSE)
			{
			LoadTexture( dd_data, ".\\Images\\Food Fight\\Ce4Crak2.pcx", &tex);

			nTextureLoaded=TRUE;			
			}
		
		fx->fScaleX = (float) 71.0;
		fx->fScaleY = (float) 71.0;
		
		DrawObjectToBrowserBuffer(dd_data, fx, &tex, SCALE_IMAGE);
		}
	break;

	case MANUAL_LOADED_EFFECT_DATA:
		
		URLData = (URLDATA *) vDataStruct;
		cpData = (EGGDATA *) URLData->ptrData;

		//for(i=0; i<(signed)(URLData->nSize/(2*sizeof(int))); i++)
		for(i=0; i<(signed)(URLData->nSize/sizeof(EGGDATA)); i++)
			{
			fx = CreateFXObject();
			if(fx==(FXOBJECT *) NULL)
				break;

			InitObject(FindEffect("Rotten-Egg"),
					fx,
					NULL,
					0,
					0);

			fx->x = cpData->x;
			fx->y = cpData->y;
			fx->nInsideBrowser	= TRUE;			
			fx->nFrame = cpData->dwFlags;
			*cpData++;
			}		

		free(URLData->ptrData);
	break;

	case MANUAL_SAVE_EFFECT_DATA:
		nTotal = GetTotalFXObjectsForEffect(FindEffect("Rotten-Egg"));
		URLData = (URLDATA *) vDataStruct;
		URLData->ptrData	= malloc(nTotal * sizeof(EGGDATA));
		URLData->nSize		= nTotal * sizeof(EGGDATA);
		cpData = (EGGDATA *) URLData->ptrData;
		

		nType = FindEffect("Rotten-Egg");

		fx = GetNextFXObject(nType, FIND_FIRST_OBJECT);
		if(fx==(FXOBJECT *) NULL)
			break;

		cpData->x = fx->x;
		cpData->y = fx->y; 
		cpData->dwFlags = fx->nFrame;
		*cpData++;

		while(1)
			{
			fx = GetNextFXObject(nType, FIND_NEXT_OBJECT);
			if(fx==(FXOBJECT *) NULL)
				break;

			cpData->x = fx->x;
			cpData->y = fx->y; 
			cpData->dwFlags = fx->nFrame;
			*cpData++;
			}
	break;
	}

return TRUE;
}



void CheckForEggCodes(void)
{
if(szCode[0]=='d' || szCode[0]=='D')
	{
	if(_stricmp(szCode, "Defense")==0)
		{
		//dfP1Defense.byObjectAlive = TRUE;
		nEffectsDefended ^= TRUE;
		if(nEffectsDefended==FALSE)
			dfP1Defense.fxObject.dwFlags = ERASE;
		ResetCode();		
		}
	}
}



void CalculateEggDefense(void)
{
int i;

switch(dfP1Defense.fxObject.nFlags)
	{
	case WAITING:  //defense is waiting for something to happen
		dfP1Defense.nDestX = nMouseX;
		dfP1Defense.nDestY = nMouseY;
		dfP1Defense.nSpeed = rand() >> 10;
		while(dfP1Defense.nSpeed>MAX_EGG_DEFENSE_SPEED)
			dfP1Defense.nSpeed = dfP1Defense.nSpeed >> 1;



		// Find out which direction the Defense should go
		if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) < 0)
			{
			if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= -MAX_EGG_DEFENSE_DISTANCE)
				dfP1Defense.nSpeed = MIN_EGG_DEFENSE_SPEED;
			}
		else if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= 0)
			{
			 if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) <= MAX_EGG_DEFENSE_DISTANCE)
				 dfP1Defense.nSpeed = MIN_EGG_DEFENSE_SPEED;
			}

		// Set the incremental value for the Defense
		dfP1Defense.fIncrX = (float) (dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2)))/dfP1Defense.nSpeed;
		dfP1Defense.fIncrY = (float) (dfP1Defense.nDestY - (dfP1Defense.fxObject.y + (img.Defenses[0].height/2)))/dfP1Defense.nSpeed;

		// Starts a Counter So we can check for the next mouse postion
		dfP1Defense.nCount=0;
		dfP1Defense.fxObject.nFlags = MOVING;
	break;

	case LOCKED:
		dfP1Defense.fxObject.x += (int) dfP1Defense.fIncrX;
		dfP1Defense.fxObject.y += (int) dfP1Defense.fIncrY;

		if(dfP1Defense.fxObject.x >= dfP1Defense.nDestX && dfP1Defense.fIncrX > 0)
			dfP1Defense.fIncrX = (float) 0.0;

		if(dfP1Defense.fxObject.x <= dfP1Defense.nDestX && dfP1Defense.fIncrX < 0)
			dfP1Defense.fIncrX = (float) 0.0;

		if(dfP1Defense.fxObject.y + 130 >= dfP1Defense.nDestY && dfP1Defense.fIncrY > 0)  //130 is a realtive size of a pie piec when it hits
			dfP1Defense.fIncrY = (float) 0.0;

		if(dfP1Defense.fxObject.y <= dfP1Defense.nDestY && dfP1Defense.fIncrY < 0)
			dfP1Defense.fIncrY = (float) 0.0;

	break;
	}



//case MOVING:

if(dfP1Defense.fxObject.nFlags!=LOCKED)
	{
	dfP1Defense.nCount++;
	dfP1Defense.fxObject.x += (int) dfP1Defense.fIncrX;
	dfP1Defense.fxObject.y += (int) dfP1Defense.fIncrY;

	if(dfP1Defense.fxObject.x >= dfP1Defense.nDestX || dfP1Defense.nCount>dfP1Defense.nSpeed)
		dfP1Defense.fxObject.nFlags = WAITING;

	if(dfP1Defense.fIncrX < 0 && dfP1Defense.fxObject.x  < nMouseX || dfP1Defense.fIncrX >= 0 && dfP1Defense.fxObject.x > nMouseX)
		dfP1Defense.fxObject.nFlags = WAITING;

	if(dfP1Defense.fIncrY < 0 && dfP1Defense.fxObject.y < nMouseY || dfP1Defense.fIncrY >= 0 && dfP1Defense.fxObject.y > nMouseY)
		dfP1Defense.fxObject.nFlags = WAITING;
	}

//break;

for(i=0; i<MAX_EGG_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==TRUE)
		{
		switch(moP1Obj[i].pfxObject->nFlags)
			{
			case FALLING:
			if(dfP1Defense.fxObject.nFlags!=LOCKED)
				{ // we want to lock onto a falling object
				dfP1Defense.nDestX = moP1Obj[i].nDestX;
				dfP1Defense.nDestY = moP1Obj[i].nDestY;
				dfP1Defense.nSpeed = rand() >> 10;
				while(dfP1Defense.nSpeed>MAX_EGG_DEFENSE_SPEED)
					dfP1Defense.nSpeed = dfP1Defense.nSpeed >> 1;



				// Find out which direction the Defense should go
				if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) < 0)
					{
					if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= -MAX_EGG_DEFENSE_DISTANCE)
						dfP1Defense.nSpeed = MIN_EGG_DEFENSE_SPEED;
					}
				else if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= 0)
					{
					 if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) <= MAX_EGG_DEFENSE_DISTANCE)
						 dfP1Defense.nSpeed = MIN_EGG_DEFENSE_SPEED;
					}

				// Set the incremental value for the Defense
				dfP1Defense.fIncrX = (float) (dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2)))/dfP1Defense.nSpeed;
				dfP1Defense.fIncrY = (float) (dfP1Defense.nDestY - (dfP1Defense.fxObject.y + (img.Defenses[0].height/2)))/dfP1Defense.nSpeed;

				// Starts a Counter So we can check for the next mouse postion
				dfP1Defense.nCount=0;
				dfP1Defense.fxObject.nFlags = LOCKED;
				}
			break;

			case EXPLODING:
			if((moP1Obj[i].pfxObject->x + 50) > dfP1Defense.fxObject.x &&  // if the img hits the defense imag at all it is blocked
				moP1Obj[i].pfxObject->x < (dfP1Defense.fxObject.x + img.Defenses[dfP1Defense.fxObject.nFrame].width-50) &&
				(moP1Obj[i].pfxObject->y + 15 )> dfP1Defense.fxObject.y &&
				moP1Obj[i].pfxObject->y < (dfP1Defense.fxObject.y + img.Defenses[dfP1Defense.fxObject.nFrame].height)-50)
				{
				PlayWave(Wave[2]);

				moP1Obj[i].pfxObject->nFlags = BLOCKED;
				moP1Obj[i].pfxObject->nFrame = 3;

				dfP1Defense.ptDef.x = (moP1Obj[i].pfxObject->x + (img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2)) - dfP1Defense.fxObject.x-95;
				dfP1Defense.ptDef.y = (moP1Obj[i].pfxObject->y + (img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2)) - dfP1Defense.fxObject.y-90;

				//Log_printf("dfP1Defense.ptDef.x:%d dfP1Defense.ptDef.y:%d\n",dfP1Defense.ptDef.x,dfP1Defense.ptDef.y);
				//DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
				//DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
				//Only paint a mark if it is within these boundaries
				if(dfP1Defense.ptDef.x >= -9 &&
					dfP1Defense.ptDef.y >= 28 &&
					dfP1Defense.ptDef.x <= 135 &&
					dfP1Defense.ptDef.y <= 105)
					DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
					
				}

			if(dfP1Defense.fxObject.nFlags==LOCKED)
				dfP1Defense.fxObject.nFlags=WAITING;
				
			break;
			}
		}
	}
}
