/*
** FF-Tomato.c - Tomato for the Food Fight Pak
**
** This Effect Pack uses the MULTIPLE DRAW Method to paint objects
**
** by Jamie Cristini
**
*/
 


#include <ddraw.h>
#include <stdio.h>
#include <windows.h>
#include <time.h>
#include <math.h>
#include "BlasterEffects.h"
#include "EffectType.h"
#include "FF-Main.h"

void CalculateTomatoShots();
void CalculateTomato(void);
int UpdateTomatoEffect(DDRAW_DATA *dd_data, int nEffect, int nMsg, int nObjectCount);
int InitEffects(void);
int DrawTomato(DDRAW_DATA *dd_data, int nMsg);
void DrawTomatoBuffer(DDRAW_DATA *dd_data, int nPlayer);
void DrawTomatoScreen(DDRAW_DATA *dd_data, int nPlayer);
void EraseTomatoBuffer(DDRAW_DATA *dd_data, int nPlayer);
int CreateTomatoShot(DDRAW_DATA *dd_data);
void CheckTomatoButtons(DDRAW_DATA *dd_data, int nPlayer);
void MoveTomatoDefense(void);
void AssignTomatoPointers(int nPlayer);
void CalculateAnTomato(MULTIOBJECT *Obj);
int TomatoManualSave(DDRAW_DATA *dd_data, int nMsg, void* vDataStruct);
void CreateTomatoPieces(MULTIOBJECT *Obj);
void CalculateTomatoPieces();
void CalculateTomatoDefense(void);
void CheckForTomatoCodes(void);


#define MAX_TOMATO_OBJECTS				5
#define MAX_TOMATO_DEFENSE_SPEED		3   // Lower number for a better defense
#define MIN_TOMATO_DEFENSE_SPEED		2	// Lower number for a better defense
#define MAX_TOMATO_DEFENSE_DISTANCE	100 // Determines the max pixels the defense can move in a single step

#define TIME_BEFORE_TOMATO_HIT_BOUNCE	-10
#define TIME_FOR_TOMATO_TO_REACH		-9



/*int InitEffects()
{
RegisterEffect("Rotten-Tomato","Food Fight", DrawTomato, UpdateTomatoEffect, ".\\images\\Food Fight\\Ce4BBlob.pcx", NULL, NULL);		
return TRUE;
}*/

int UpdateTomatoEffect(DDRAW_DATA *dd_data, int nEffect, int nMsg, int nObjectCount)
{
int i;

switch(nMsg)
	{
	case EFFECT_STARTED:
		/* Tomato */
		if(FindEffect("Rotten-Tomato") == nEffect)
			{
			//Load Animated Cursor
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Cc4Toma1.pcx"  , &img.AnimCur[0]);

			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma1.pcx", &img.WeaponEnd[0]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma2.pcx", &img.WeaponEnd[1]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma3.pcx", &img.WeaponEnd[2]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma4.pcx", &img.WeaponEnd[3]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma5.pcx", &img.WeaponEnd[4]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma6.pcx", &img.WeaponEnd[5]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Co4Toma7.pcx", &img.WeaponEnd[6]);

			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4BBlob.pcx", &img.WeaponEnd[7]);

			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop1.pcx", &img.WeaponEnd[8]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop2.pcx", &img.WeaponEnd[9]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop3.pcx", &img.WeaponEnd[10]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop4.pcx", &img.WeaponEnd[11]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop5.pcx", &img.WeaponEnd[12]);
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Drop6.pcx", &img.WeaponEnd[13]);

			LoadTexture(dd_data, ".\\Images\\Food Fight\\Ce4Tomt1.pcx", &img.WeaponEnd[14]);

			// Load Defense image
			LoadTexture(dd_data, ".\\Images\\Food Fight\\Cd4Tray.pcx", &img.Defenses[0]);

			Wave[0] = LoadWave(".\\Sounds\\Food Fight\\NoAmmo.wav");
			Wave[1] = LoadWave(".\\Sounds\\Food Fight\\tomato3.wav");
			Wave[2] = LoadWave(".\\Sounds\\Food Fight\\tomatot.wav");


			InitObject(-3, 
				&dfP1Defense.fxObject, 
				&img.Defenses[0],
				nMouseX - (img.Defenses[0].width / 2),
				nMouseY - (img.Defenses[0].height/2));					

			srand( (unsigned)time( NULL ) );


			for(i=0; i<MAX_TOMATO_OBJECTS; i++)
				{
				moP1Obj[i].byObjectAlive=FALSE;
				moP1Obj[i].byID = i;
				}

			for(i=0; i<MAX_PIECES; i++)
				{
				mdDrip[i].byObjectAlive=FALSE;
				mdDrip[i].byID = i;
				}

			break;
			}
		break;

	case EFFECT_ENDED:
		if(FindEffect("Rotten-Tomato") == nEffect)
			{
			DeleteWaves();

			DeleteTexture(&img.AnimCur[0]);

			//for(i=0; i<12; i++)
			//	DeleteTexture(&img.Weapon[i]);

			for(i=0; i<15; i++)
				DeleteTexture(&img.WeaponEnd[i]);

			for(i=0; i<1; i++)
				DeleteTexture(&img.Defenses[i]);

			for(i=0; i<MAX_TOMATO_OBJECTS; i++)
				{
				if(moP1Obj[i].byObjectAlive==TRUE)
					{
					moP1Obj[i].byObjectAlive=FALSE;
					moP1Obj[i].byID = i;
					moP1Obj[i].pfxObject->nFlags=0;
					DeleteLastFXObject();
					}
				}


			break;
			}
	break;


	case URL_CHANGED:

	break;
	}

return TRUE;
}

int DrawTomato(DDRAW_DATA *dd_data, int nMsg)
{
dfP1Defense.byObjectAlive = nEffectsDefended;

switch(nMsg)
	{
	case DRAW_EFFECT:
		//AssignTomatoPointers(PLAYER_ONE);
		CalculateTomato();
		DrawTomatoBuffer(dd_data, PLAYER_ONE);
		DrawTomatoScreen(dd_data, PLAYER_ONE);
		EraseTomatoBuffer(dd_data, PLAYER_ONE);
	break;

	case CALCULATE_AND_SKIP_FRAME:
		//AssignTomatoPointers(PLAYER_ONE);
		CalculateTomato();
	break;
	}


CheckTomatoButtons(dd_data, PLAYER_ONE);
return TRUE;
}

void CalculateTomato()
{
fxPlayerOne.x = nMouseX - (img.AnimCur[fxPlayerOne.nFrame].width/2);
fxPlayerOne.y = nMouseY - (img.AnimCur[fxPlayerOne.nFrame].height/2);

CalculateTomatoShots();
CalculateTomatoPieces();
}

void CalculateTomatoShots()
{
int i;

for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==TRUE)
		{
		switch(moP1Obj[i].pfxObject->nFlags)
			{
			case FALLING:
				
				moP1Obj[i].pfxObject->nFrameTime++;
				if(moP1Obj[i].pfxObject->nFrameTime>1)
					{
					moP1Obj[i].pfxObject->nFrameTime=0;
					moP1Obj[i].pfxObject->nFrame++;
					if(moP1Obj[i].pfxObject->nFrame>HIWORD(moP1Obj[i].pfxObject->dwFlags2))
						moP1Obj[i].pfxObject->nFrame = LOWORD(moP1Obj[i].pfxObject->dwFlags2);

					}

				/*moP1Obj[i].pfxObject->fAngle += 10;
				if(moP1Obj[i].pfxObject->fAngle>360)
					moP1Obj[i].pfxObject->fAngle = (float) 0.0;*/

				moP1Obj[i].pfxObject->fScaleY -= 5;
				moP1Obj[i].pfxObject->fScaleX -= 5;

				moP1Obj[i].pfxObject->x = (int) (moP1Obj[i].nStartX - moP1Obj[i].pfxObject->fScaleX/2);
				moP1Obj[i].pfxObject->y = (int) ((moP1Obj[i].nStartY + moP1Obj[i].d) - moP1Obj[i].pfxObject->fScaleY/2); 


				if(moP1Obj[i].t<-11)   // if TRUE then the Molotov has hit
					{
					PlayWave(Wave[1]);
									
					moP1Obj[i].pfxObject->nFlags = HIT;
					moP1Obj[i].pfxObject->nFrame=7;
					moP1Obj[i].nDrawFlags = 0;//SCALE_IMAGE;  //fSCaleX = 71.0 when hits

					moP1Obj[i].pfxObject->x = (int) (moP1Obj[i].nStartX - img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2);
					moP1Obj[i].pfxObject->y = (int) ((moP1Obj[i].nStartY + moP1Obj[i].d) - img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].height/2);

					CreateTomatoPieces(&moP1Obj[i]);
					}
				else
					{
					moP1Obj[i].t += nFrameIncr;  // Increment Time

					moP1Obj[i].d = (float) (.5 * moP1Obj[i].g * (moP1Obj[i].t * moP1Obj[i].t));  // Find the new distance
					}
			break;

			case HIT:
			break;
			}
		}
	}

if(dfP1Defense.byObjectAlive==TRUE)
	CalculateTomatoDefense();

}

void DrawTomatoBuffer(DDRAW_DATA *dd_data, int nPlayer)
{
AssignTomatoPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	DrawObjectToBuffer(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame], 0);	


for(i=0; i<MAX_PIECES; i++)
	{
	if(mdDrip[i].byObjectAlive==TRUE)
		{
		DrawObjectToBuffer(dd_data, &mdDrip[i].fxObject, &img.WeaponEnd[mdDrip[i].fxObject.nFrame], mdDrip[i].nFlags);	
		}

	}


for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case BLOCKED:
			break;

			case HIT:
				DrawObjectToBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	
				DrawObjectToScreenCapture(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	
			break;

			default:
				DrawObjectToBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame], pmoObj[i].nDrawFlags);	

			}
		}
	}


// draw the weapon
DrawObjectToBuffer(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame], 0);	

}

void DrawTomatoScreen(DDRAW_DATA *dd_data, int nPlayer)
{
AssignTomatoPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	DrawBufferObjectToScreen(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	
if(dfP1Defense.fxObject.dwFlags==ERASE)
	EraseObjectOnScreen(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	


/*for(i=0; i<MAX_PIECES; i++)
	{
	if(mdDrip[i].byObjectAlive==TRUE)
		{
		DrawBufferObjectToScreen(dd_data, &mdDrip[i].fxObject, &img.WeaponEnd[mdDrip[i].fxObject.nFrame]);	
		}
	}*/


for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case BLOCKED:
				EraseObjectOnScreen(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
				moP1Obj[i].pfxObject->x = -9999;
				moP1Obj[i].pfxObject->y = -9999;
				moP1Obj[i].byObjectAlive=FALSE;
				moP1Obj[i].pfxObject->nFlags=0;
			break;

			default:
				DrawBufferObjectToScreen(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
			}

		}
	}

for(i=0; i<MAX_PIECES; i++)
	{
	if(mdDrip[i].byObjectAlive==TRUE)
		{
		DrawBufferObjectToScreen(dd_data, &mdDrip[i].fxObject, &img.WeaponEnd[mdDrip[i].fxObject.nFrame]);	
		}
	}

//draw the weapon
DrawBufferObjectToScreen(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame]);	


}

void EraseTomatoBuffer(DDRAW_DATA *dd_data, int nPlayer)
{
AssignTomatoPointers(nPlayer);

if(dfP1Defense.byObjectAlive==TRUE)
	EraseObjectInBuffer(dd_data, &dfP1Defense.fxObject, &img.Defenses[dfP1Defense.fxObject.nFrame]);	

for(i=0; i<MAX_PIECES; i++)
	{
	if(mdDrip[i].byObjectAlive==TRUE)
		{
		EraseObjectInBuffer(dd_data, &mdDrip[i].fxObject, &img.WeaponEnd[mdDrip[i].fxObject.nFrame]);	
		}
	}


for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(pmoObj[i].byObjectAlive==TRUE)
		{
		switch(pmoObj[i].pfxObject->nFlags)
			{
			case BLOCKED:
			break;

			case HIT:
				EraseObjectInBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
				pmoObj[i].byObjectAlive=FALSE;
				moP1Obj[i].pfxObject->nFlags=0;
			break;

			default:
				EraseObjectInBuffer(dd_data, pmoObj[i].pfxObject, &img.WeaponEnd[pmoObj[i].pfxObject->nFrame]);	
			}
		}
	}

//erase the weapon
EraseObjectInBuffer(dd_data, pfxPlayer, &img.AnimCur[pfxPlayer->nFrame]);	


}


void CheckTomatoButtons(DDRAW_DATA *dd_data, int nPlayer)
{
//AssignTomatoPointers(nPlayer);

/* UPDATE MOUSE BUTTONS */
if(LButtonPressed==TRUE)
	{
	LButtonPressed=FALSE;					
	
	if(CreateTomatoShot(dd_data)==TRUE)
		{
		nFrameIncr= -1;		// We want to increment time by a -1
		}
	}
 
if(RButtonPressed==TRUE)
	{
	RButtonPressed=FALSE;					
	dfP1Defense.byObjectAlive^=TRUE;
	}

CheckForTomatoCodes();
}


int CreateTomatoShot(DDRAW_DATA *dd_data)
{
int i;

for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==FALSE)
		break;

	if(i == (MAX_TOMATO_OBJECTS-1))
		return FALSE;
	}

if((moP1Obj[i].pfxObject = (FXOBJECT *) CreateFXObject()) == (FXOBJECT *) NULL)
	{
	PlayWave(Wave[0]);
	return FALSE;
	}

InitObject(FindEffect("Rotten-Tomato"),
		   moP1Obj[i].pfxObject,
		   &img.Weapon[0],
		   nMouseX - (img.WeaponEnd[0].width/2),
		   nScreenHeight - (img.WeaponEnd[0].height/2));

moP1Obj[i].pfxObject->fScaleX = (float) 256;
moP1Obj[i].pfxObject->fScaleY = (float) 256;
moP1Obj[i].pfxObject->nFlags = FALLING;
moP1Obj[i].nDrawFlags = SCALE_IMAGE|ROTATE_IMAGE;


moP1Obj[i].nStartX = nMouseX; 
moP1Obj[i].nStartY = (int) (nMouseY-((nScreenHeight - nMouseY) * .33));  // .33 just looks good
moP1Obj[i].nDestX = nMouseX - (img.AnimCur[0].width/2);
moP1Obj[i].nDestY = nMouseY - (img.AnimCur[0].height/2);


moP1Obj[i].d = (float) (nScreenHeight - moP1Obj[i].nStartY); // the total distance
moP1Obj[i].t = (float) 24.0;				  // 24 frames until we hit
moP1Obj[i].g = (float) (moP1Obj[i].d / (.5 * (moP1Obj[i].t * moP1Obj[i].t))); // g = d / (.5 * t^2)

moP1Obj[i].pfxObject->dwFlags2 = MAKELONG(0, 6);
moP1Obj[i].byObjectAlive=TRUE;

return TRUE;
}

void MoveTomatoDefense(void)
{

}


void AssignTomatoPointers(int nPlayer)
{
switch(nPlayer)
	{
	case PLAYER_ONE:
		pfxPlayer = (FXOBJECT *) &fxPlayerOne;
		pfxWeapon = (FXOBJECT *) &fxP1Weapon;
		pmoObj =	(MULTIOBJECT *) &moP1Obj[0];
	break;

	case PLAYER_TWO:
	break;

	default:
		pfxWeapon = (FXOBJECT *) NULL;
		pmoObj = (MULTIOBJECT *) NULL;
	}
}





void CalculateTomatoPieces()
{
int i;

for(i=0; i<MAX_PIECES; i++)
	{
	if(mdDrip[i].byObjectAlive==TRUE)
		{
		if(mdDrip[i].fxObject.x > mdDrip[i].nStartX - 100)
			mdDrip[i].fxObject.x += (int) mdDrip[i].fIncrX;

		if(mdDrip[i].fxObject.nFlags>50)
			{
			mdDrip[i].fIncrX += (float) .5;
			if(mdDrip[i].fIncrX>0) 
				mdDrip[i].fIncrX=(float) 0.0; 

			if(mdDrip[i].fxObject.fAngle < 85)
				mdDrip[i].fxObject.fAngle += 5;

			}
		else
			{
			mdDrip[i].fIncrX += (float) -.5;
			if(mdDrip[i].fIncrX<0) 
				mdDrip[i].fIncrX=(float)0.0; 

			if(mdDrip[i].fxObject.fAngle > 85)
				mdDrip[i].fxObject.fAngle += -5;

			}
		

		//if((mdDrip[i].fxObject.y + img.WeaponEnd[mdDrip[i].fxObject.nFrame].height)> nScreenHeight)
		if(mdDrip[i].fxObject.y > (nScreenHeight+(nScreenHeight>>2)))
			mdDrip[i].byObjectAlive=FALSE;

		//Log_printf("Tomato Piece falling before(%d)= x:%d y:%d\n", mdDrip[i].byID, mdDrip[i].fxObject.x, mdDrip[i].fxObject.y);
		mdDrip[i].fxObject.y = (int) ((mdDrip[i].nStartY + mdDrip[i].d) - (img.WeaponEnd[mdDrip[i].fxObject.nFrame].height/2));
		//Log_printf("Tomato Piece falling after(%d)= x:%d y:%d\n", mdDrip[i].byID, mdDrip[i].fxObject.x, mdDrip[i].fxObject.y);

		mdDrip[i].t += mdDrip[i].fIncrY;  // Increment Time

		mdDrip[i].d = (float) (.5 * mdDrip[i].g * (mdDrip[i].t * mdDrip[i].t));  // Find the new distance
		

		//mdDrip[i].fxObject.y += 20;
		}
	}
}



void CreateTomatoPieces(MULTIOBJECT *Obj)
{
int nRandom=0;
int ii;
int nCount=3;
int nIndex=0;
int i;


nRandom = rand() >> 8;
while(nRandom>4)
	nRandom = nRandom >> 1;

nCount = nRandom;
if(nCount<1)
	nCount = 2;
	
for(ii=0; ii<nCount; ii++)
	{
	for(i=nIndex; i<MAX_PIECES; i++)
		{
		if(mdDrip[i].byObjectAlive==FALSE)
			break;
			
		if(i==(MAX_PIECES-1))
			return;
		}

	nIndex = i;

	if(mdDrip[i].byObjectAlive == FALSE)
		{
		InitObject(-1, &mdDrip[i].fxObject, (TEXTURE *) NULL, 0, 0);

		mdDrip[i].byObjectAlive = TRUE;
		nRandom = rand() % 30;
		mdDrip[i].nStartX = Obj->nDestX + nRandom;
		nRandom = rand() % 20;
		mdDrip[i].nStartY = Obj->nDestY + nRandom;

		mdDrip[i].fIncrY = (float) 1.0;

		mdDrip[i].fxObject.nFlags = rand() % 100;


		mdDrip[i].fxObject.x = Obj->nDestX;
		mdDrip[i].fxObject.y = Obj->nDestY;

		mdDrip[i].fxObject.nFrame = rand() >> 10;
		while(mdDrip[i].fxObject.nFrame > 6)
			mdDrip[i].fxObject.nFrame = mdDrip[i].fxObject.nFrame >> 1;
		mdDrip[i].fxObject.nFrame += 7;
		if(mdDrip[i].fxObject.nFrame==7)
			mdDrip[i].fxObject.nFrame++;


		mdDrip[i].d = (float) nScreenHeight;
		mdDrip[i].t = (float) -24.0;							  // 24 frames until we hit
		mdDrip[i].g = (float) (mdDrip[i].d / (.5 * (mdDrip[i].t * mdDrip[i].t))); // g = d / (.5 * t^2)
		mdDrip[i].t = (float) -5.0;							  // 24 frames until we hit
		mdDrip[i].d = (float) (nScreenHeight - mdDrip[i].nStartY);

		if(mdDrip[i].fxObject.nFlags > 50)
			{
			mdDrip[i].fIncrX = (float) -10.0;
			mdDrip[i].fxObject.fAngle = (float) 0.0;
			nRandom = rand() % 30;
			mdDrip[i].fxObject.x += -nRandom;
			nRandom = rand() % 20;
			mdDrip[i].fxObject.y += nRandom;

			mdDrip[i].nFlags = ROTATE_IMAGE;
			}
		else
			{
			mdDrip[i].fIncrX = (float) 10.0;
			mdDrip[i].fxObject.fAngle = (float) 185.0;
			nRandom = rand() % 30;
			mdDrip[i].fxObject.x += nRandom + img.WeaponEnd[mdDrip[i].fxObject.nFrame].width;
			nRandom = rand() % 20;
			mdDrip[i].fxObject.y += nRandom;

			mdDrip[i].nFlags = ROTATE_IMAGE|FLIP_VERTICAL;
			}

		mdDrip[i].d = (float) (.5 * mdDrip[i].g * (mdDrip[i].t * mdDrip[i].t));
		//Log_printf("Tomato Piece Start (%d) x:%d y:%d\n", mdDrip[i].byID, mdDrip[i].fxObject.x, mdDrip[i].fxObject.y);
		}
	}
}





void CheckForTomatoCodes(void)
{
if(szCode[0]=='d' || szCode[0]=='D')
	{
	if(_stricmp(szCode, "Defense")==0)
		{
		//dfP1Defense.byObjectAlive = TRUE;
		nEffectsDefended ^= TRUE;
		if(nEffectsDefended==FALSE)
			dfP1Defense.fxObject.dwFlags = ERASE;
		ResetCode();		
		}
	}
}



void CalculateTomatoDefense(void)
{
int i;

switch(dfP1Defense.fxObject.nFlags)
	{
	case WAITING:  //defense is waiting for something to happen
		dfP1Defense.nDestX = nMouseX;
		dfP1Defense.nDestY = nMouseY;
		dfP1Defense.nSpeed = rand() >> 10;
		while(dfP1Defense.nSpeed>MAX_TOMATO_DEFENSE_SPEED)
			dfP1Defense.nSpeed = dfP1Defense.nSpeed >> 1;



		// Find out which direction the Defense should go
		if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) < 0)
			{
			if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= -MAX_TOMATO_DEFENSE_DISTANCE)
				dfP1Defense.nSpeed = MIN_TOMATO_DEFENSE_SPEED;
			}
		else if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= 0)
			{
			 if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) <= MAX_TOMATO_DEFENSE_DISTANCE)
				 dfP1Defense.nSpeed = MIN_TOMATO_DEFENSE_SPEED;
			}

		// Set the incremental value for the Defense
		dfP1Defense.fIncrX = (float) (dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2)))/dfP1Defense.nSpeed;
		dfP1Defense.fIncrY = (float) (dfP1Defense.nDestY - (dfP1Defense.fxObject.y + (img.Defenses[0].height/2)))/dfP1Defense.nSpeed;

		// Starts a Counter So we can check for the next mouse postion
		dfP1Defense.nCount=0;
		dfP1Defense.fxObject.nFlags = MOVING;
	break;

	case LOCKED:
		dfP1Defense.fxObject.x += (int) dfP1Defense.fIncrX;
		dfP1Defense.fxObject.y += (int) dfP1Defense.fIncrY;

		if(dfP1Defense.fxObject.x >= dfP1Defense.nDestX && dfP1Defense.fIncrX > 0)
			dfP1Defense.fIncrX = (float) 0.0;

		if(dfP1Defense.fxObject.x <= dfP1Defense.nDestX && dfP1Defense.fIncrX < 0)
			dfP1Defense.fIncrX = (float) 0.0;

		if(dfP1Defense.fxObject.y + 130 >= dfP1Defense.nDestY && dfP1Defense.fIncrY > 0)  //130 is a realtive size of a pie piec when it hits
			dfP1Defense.fIncrY = (float) 0.0;

		if(dfP1Defense.fxObject.y <= dfP1Defense.nDestY && dfP1Defense.fIncrY < 0)
			dfP1Defense.fIncrY = (float) 0.0;

	break;
	}



//case MOVING:
if(dfP1Defense.fxObject.nFlags!=LOCKED)
	{
	dfP1Defense.nCount++;
	dfP1Defense.fxObject.x += (int) dfP1Defense.fIncrX;
	dfP1Defense.fxObject.y += (int) dfP1Defense.fIncrY;

	if(dfP1Defense.fxObject.x >= dfP1Defense.nDestX || dfP1Defense.nCount>dfP1Defense.nSpeed)
		dfP1Defense.fxObject.nFlags = WAITING;

	if(dfP1Defense.fIncrX < 0 && dfP1Defense.fxObject.x  < nMouseX || dfP1Defense.fIncrX >= 0 && dfP1Defense.fxObject.x > nMouseX)
		dfP1Defense.fxObject.nFlags = WAITING;

	if(dfP1Defense.fIncrY < 0 && dfP1Defense.fxObject.y < nMouseY || dfP1Defense.fIncrY >= 0 && dfP1Defense.fxObject.y > nMouseY)
		dfP1Defense.fxObject.nFlags = WAITING;
	}
//break;

for(i=0; i<MAX_TOMATO_OBJECTS; i++)
	{
	if(moP1Obj[i].byObjectAlive==TRUE)
		{
		switch(moP1Obj[i].pfxObject->nFlags)
			{
			case FALLING:
			if(dfP1Defense.fxObject.nFlags!=LOCKED)
				{ // we want to lock onto a falling object
				dfP1Defense.nDestX = moP1Obj[i].nDestX;
				dfP1Defense.nDestY = moP1Obj[i].nDestY;
				dfP1Defense.nSpeed = rand() >> 10;
				while(dfP1Defense.nSpeed>MAX_TOMATO_DEFENSE_SPEED)
					dfP1Defense.nSpeed = dfP1Defense.nSpeed >> 1;



				// Find out which direction the Defense should go
				if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) < 0)
					{
					if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= -MAX_TOMATO_DEFENSE_DISTANCE)
						dfP1Defense.nSpeed = MIN_TOMATO_DEFENSE_SPEED;
					}
				else if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) >= 0)
					{
					 if((dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2))) <= MAX_TOMATO_DEFENSE_DISTANCE)
						 dfP1Defense.nSpeed = MIN_TOMATO_DEFENSE_SPEED;
					}

				// Set the incremental value for the Defense
				dfP1Defense.fIncrX = (float) (dfP1Defense.nDestX - (dfP1Defense.fxObject.x + (img.Defenses[0].width/2)))/dfP1Defense.nSpeed;
				dfP1Defense.fIncrY = (float) (dfP1Defense.nDestY - (dfP1Defense.fxObject.y + (img.Defenses[0].height/2)))/dfP1Defense.nSpeed;

				// Starts a Counter So we can check for the next mouse postion
				dfP1Defense.nCount=0;
				dfP1Defense.fxObject.nFlags = LOCKED;
				}
			break;

			case HIT:
			if((moP1Obj[i].pfxObject->x + img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width) > dfP1Defense.fxObject.x &&  // if the img hits the defense imag at all it is blocked
				moP1Obj[i].pfxObject->x < (dfP1Defense.fxObject.x + img.Defenses[dfP1Defense.fxObject.nFrame].width) &&
				(moP1Obj[i].pfxObject->y + img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].height) > dfP1Defense.fxObject.y &&
				moP1Obj[i].pfxObject->y < (dfP1Defense.fxObject.y + img.Defenses[dfP1Defense.fxObject.nFrame].height))
				{
				PlayWave(Wave[2]);

				moP1Obj[i].pfxObject->nFlags = BLOCKED;
				moP1Obj[i].pfxObject->nFrame = 14;

				dfP1Defense.ptDef.x = (moP1Obj[i].pfxObject->x + (img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2)) - dfP1Defense.fxObject.x - 65;
				dfP1Defense.ptDef.y = (moP1Obj[i].pfxObject->y + (img.WeaponEnd[moP1Obj[i].pfxObject->nFrame].width/2)) - dfP1Defense.fxObject.y - 65;

				//Log_printf("dfP1Defense.ptDef.x:%d dfP1Defense.ptDef.y:%d\n",dfP1Defense.ptDef.x,dfP1Defense.ptDef.y);
				//DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
				//DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
				//Only paint a mark if it is within these boundaries
				if(dfP1Defense.ptDef.x >= 17 &&
					dfP1Defense.ptDef.y >= 32 &&
					dfP1Defense.ptDef.x <= 104 &&
					dfP1Defense.ptDef.y <= 95)
					DrawTextureInTexture(&img.Defenses[0], &img.WeaponEnd[moP1Obj[i].pfxObject->nFrame], &dfP1Defense.ptDef);
					
				}

			if(dfP1Defense.fxObject.nFlags==LOCKED)
				dfP1Defense.fxObject.nFlags=WAITING;
				
			break;
			}
		}
	}
}
