/*******************************************************************************
*
*  Web Vengeance Poo Pack - GST.c
*
*  The Poo Pack's Giant Steaming Turd effect routines.
*
*	Copyright (c) 1997 SegaSoft, Inc.  All Rights Reserved.
*
*------------------------------------------------------------------------------
*                   CONFIDENTIAL AND PROPRIETARY INFORMATION
*------------------------------------------------------------------------------
*  The contents of this file are the property of SegaSoft, Inc., 
*  and may not be used or duplicated without the permission of SegaSoft, Inc.
*------------------------------------------------------------------------------
*
*  Edit History:
*		97-Oct-07	DJS	Creation.
*
*  Initials:
*  	DJS	Dan J. Su
*
*******************************************************************************/
#include <ddraw.h>
#include <windows.h>
#include <stdlib.h>
#include "BlasterEffects.h"
#include "EffectType.h"

#define DEFINE_GST
#include "GST.h"
#include "main.h"


// ---------- Defines
#define MAX_OBJECTS_GST          ( 3UL )  // Can only throw this many GST's at a time
#define MIN_SPLATTER_PER_GST     ( 4 )    // Minimum # of splatter objects created for each GST impact
#define MAX_SPLATTER_PER_GST     ( 7 )    // Maximum # of splatter objects created for each GST impact
#define MAX_OBJECTS_SPLATTER     ( MAX_OBJECTS_GST * MAX_SPLATTER_PER_GST )

   // Gravity ( acceleration downward ) for each GST, in pixels per vblank per vblank
   // Each vblank is approx. 1/30th of a second
#define G_ACCELERATION           (  4 )
#define PI                       ( 3.1415926535f )
#define GST_STARTING_Y           ( nScreenHeight - s_aTextures[ TEXTURE_GST_0 ].height )
#define NUM_OF_FRAME_DELAYS      (  5 )   // Number of different frame delay values
#define DEFAULT_ANIM_FRAME_DELAY (  2 )   // Draw a new frame every ANIM_FRAME_DELAY vblanks
                                          // Note that each GST uses it's own frame delay.
#define GST_LAUNCH_DELAY         ( 400 )  // Length of grunt sound, in milliseconds

   // ----- Texture indexes
#define TEXTURE_CURSOR_0         (  0 )   // Animated GST cursor
#define TEXTURE_CURSOR_1         (  1 )
#define TEXTURE_CURSOR_2         (  2 )
#define TEXTURE_CURSOR_3         (  3 )
#define TEXTURE_GST_0            (  4 )   // First frame of GST being thrown in the air
#define TEXTURE_GST_1            (  5 )
#define TEXTURE_GST_2            (  6 )
#define TEXTURE_GST_3            (  7 )
#define TEXTURE_GST_4            (  8 )   // Last frame of GST before it hits the page
#define TEXTURE_FINAL_0          (  9 )   // 3 different final GST images
#define TEXTURE_FINAL_1          ( 10 )   
#define TEXTURE_FINAL_2          ( 11 )
#define TEXTURE_SPLATTER_0       ( 12 )   // Pieces of splatter, traveling to the right
#define TEXTURE_SPLATTER_1       ( 13 )   // We can rotate these pieces as they fall
#define TEXTURE_SPLATTER_2       ( 14 )   // depending on their current trajectory
#define TEXTURE_SPLATTER_3       ( 15 )
#define TEXTURE_SPLATTER_4       ( 16 )
#define TEXTURE_SPLATTER_5       ( 17 )
#define TEXTURE_DEFENSE          ( 18 )   // Umbrella defense
#define TEXTURE_DEFENSE_DAMAGE_0 ( 19 )   // 3 different GST stains on umbrella
#define TEXTURE_DEFENSE_DAMAGE_1 ( 20 )   
#define TEXTURE_DEFENSE_DAMAGE_2 ( 21 )
#define NUM_TEXTURES_TOTAL       ( 22 )

   // ----- Sound indexes
#define SOUND_GRUNT              ( 0 )
#define SOUND_FART_0             ( 1 )    // Sound when GST is launched
#define SOUND_FART_1             ( 2 )    // Second fart sound
#define SOUND_HIT_PAGE_0         ( 3 )    // Sound of GST hitting the page
#define SOUND_HIT_PAGE_1         ( 4 )    // Sound of GST hitting the page
#define SOUND_HIT_PAGE_2         ( 5 )    // Sound of GST hitting the page
#define SOUND_BLOCKED            ( 6 )    // GST blocked by umbrella
#define NUM_SOUNDS_TOTAL         ( 7 )

   // ----- Defense info
#define DEFENSE_STATE_SEEKING    ( 1 )    // Defense mechanism needs to find a new target, cursor or GST
#define DEFENSE_STATE_MOVING     ( 2 )    // Defense mechanism is moving towards target
#define DEFENSE_FOLLOW_CURSOR    ( -999 ) // Pick an invalid x-coordinate to indicate
                                          // there are no GSTs to target
#define DEFENSE_MIN_STEP         ( 2 )    // Minimum # steps to reach destination
#define DEFENSE_MAX_STEP         ( 5 )    // Maximum # of steps to reach destination

   // Collision box offsets, relative to top-left corner of the TEXTURE_FINAL_0 bitmap
   // This area should be relatively small and centered where the GST would hit the page
#define GST_COLLISION_BOX_X1     (   5 )
#define GST_COLLISION_BOX_X2     ( 250 )
#define GST_COLLISION_BOX_Y1     (  21 )
#define GST_COLLISION_BOX_Y2     ( 230 )

   // ----- States for the GST throwing sequence
#define GST_STATE_INVALID        ( 0 )    // Marks an unused GST object
#define GST_STATE_WAITING        ( 1 )    // Waiting for grunt sound to finish
#define GST_STATE_THROWN         ( 2 )    // GST has been thrown in the air
#define GST_STATE_HIT_PAGE       ( 3 )    // GST is about to leave a mark on the web page
#define GST_STATE_BLOCKED        ( 4 )    // GST has been blocked

   // ----- States for the splatter objects
#define SPLATTER_STATE_INVALID   ( 0 )    // This splatter object is not being used
#define SPLATTER_STATE_ALIVE     ( 1 )    // This splatter object is being drawn on the screen
#define SPLATTER_STATE_ERASE     ( 2 )    // Erase splatter from screen and kill the object

// ---------- Structures and Types
   // Each GST object simply contains the FXOBJECT for the GST, 
   // which is also used for the GST and final image on the page.
typedef struct gst_s
{
   DWORD       dwState;    // Current state we're in.  See GST_STATE_... defines above.
   DWORD       dwStartTime; // Wait a bit for the grunt sound to finish before
                            // we actually shoot out the GST with a fart sound.
   FXOBJECT *  pfxGST;     // Pointer to FXOBJECT in main app's global Projectile[] array
                           // This is the hole, crater, etc. that gets saved for the current web page.
                           // nFlags member is used to store y-velocity, remember to clear for each saved image!
                           // dwFlags member is used to track the final x position
                           // dwFlags2 member is used to track the final y position
	WORD        wFrameDelay;   // Number of vblanks in between animation frames
                              // The lower the target, the shorter the delay and hence
                              // the sooner the GST will hit the target.
                              // This helps us shape the parabolic trajectory of the GST so it looks nice :)
   WORD        wPad;
} gst_t;

   // Splatter objects which come from the GST's impact with the page.
typedef struct splatter_s
{
   DWORD    dwState;       // Current state of this splatter object, basically, is it alive or not?
   FXOBJECT fxObject;      // Sprite drawing info:
                           // dwFlags contains current x-velocity
                           // dwFlags2 contains current y-velocity
                           // fAngle is calculated from the x- and y-velocities
} splatter_t;


// ---------- Local Functions
static void GST_CheckInputDevices         ( void );
static void GST_CheckCheatCodes           ( void );
static void GST_Start                     ( void );
static void GST_CreateSplatter            ( int x, int y );
static void GST_Calculate                 ( void );
static void GST_DrawBuffer                ( DDRAW_DATA *pDDrawData );
static void GST_EraseBuffer               ( DDRAW_DATA *pDDrawData );
static void GST_DrawScreen                ( DDRAW_DATA *pDDrawData );
static void GST_DefenseMove               ( void );
static void GST_DefenseCheckForCollision  ( void );


// ---------- Local Variables
static gst_t      s_aGST[ MAX_OBJECTS_GST ];             // Array of GST objects
static splatter_t s_aSplatter[ MAX_OBJECTS_SPLATTER ];   // Splatter objects
static TEXTURE    s_aTextures[ NUM_TEXTURES_TOTAL ];     // Textures
static int        s_aiSounds[ NUM_SOUNDS_TOTAL ];        // Sound indexes
static WORD       s_awFrameDelays[ NUM_OF_FRAME_DELAYS + 1 ] = 
                  { 4, 4, 3, 3, 3, 2 };


/******************************************************************************
*
*  NAME
*     GST_Update
*
*  SYNOPSIS
*     iSuccess = GST_Update( pDDrawData, iEffect, iMsg, iObjectCount );
*
*  DESCRIPTION
*     Starts or ends the GST effect.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iEffect        = Effect ID, should be the ID for the GST effect
*     int            iMsg           = Message to process i.e. start or end effect
*     int            iObjectCount   = Not used
*
*  OUTPUTS
*		int iSuccess = TRUE if update was successful, FALSE otherwise
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL GST_Update( DDRAW_DATA *pDDrawData, int iEffect, int iMsg, int iObjectCount )
{
   if ( FindEffect( EFFECT_NAME_GST ) != iEffect )
      return ( FALSE );

   switch( iMsg )
   {
      case EFFECT_STARTED:
      {
         srand( GetTickCount() );

         // Load sounds
			s_aiSounds[ SOUND_GRUNT ]        = LoadWave( ".\\Sounds\\Poo\\GST\\Grunt.wav" );
			s_aiSounds[ SOUND_FART_0 ]       = LoadWave( ".\\Sounds\\Poo\\GST\\Fart.wav" );
			s_aiSounds[ SOUND_FART_1 ]       = LoadWave( ".\\Sounds\\Poo\\GST\\Fart2.wav" );
         s_aiSounds[ SOUND_HIT_PAGE_0 ]   = LoadWave( ".\\Sounds\\Poo\\GST\\Plop.wav" );
         s_aiSounds[ SOUND_HIT_PAGE_1 ]   = LoadWave( ".\\Sounds\\Poo\\GST\\Plop2.wav" );
         s_aiSounds[ SOUND_HIT_PAGE_2 ]   = LoadWave( ".\\Sounds\\Poo\\GST\\Plop3.wav" );
         s_aiSounds[ SOUND_BLOCKED ]      = LoadWave( ".\\Sounds\\Poo\\GST\\Blocked.wav" );

         // Load textures
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\Cursor1.pcx", &s_aTextures[ TEXTURE_CURSOR_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\Cursor2.pcx", &s_aTextures[ TEXTURE_CURSOR_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\Cursor3.pcx", &s_aTextures[ TEXTURE_CURSOR_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\Cursor4.pcx", &s_aTextures[ TEXTURE_CURSOR_3 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7ofly01.pcx", &s_aTextures[ TEXTURE_GST_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7ofly02.pcx", &s_aTextures[ TEXTURE_GST_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7ofly03.pcx", &s_aTextures[ TEXTURE_GST_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7ofly04.pcx", &s_aTextures[ TEXTURE_GST_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7ofly05.pcx", &s_aTextures[ TEXTURE_GST_4 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturda.pcx",   &s_aTextures[ TEXTURE_FINAL_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturdb.pcx",   &s_aTextures[ TEXTURE_FINAL_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturdc.pcx",   &s_aTextures[ TEXTURE_FINAL_2 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop01.pcx",   &s_aTextures[ TEXTURE_SPLATTER_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop02.pcx",   &s_aTextures[ TEXTURE_SPLATTER_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop03.pcx",   &s_aTextures[ TEXTURE_SPLATTER_2 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop04.pcx",   &s_aTextures[ TEXTURE_SPLATTER_3 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop05.pcx",   &s_aTextures[ TEXTURE_SPLATTER_4 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7edrop06.pcx",   &s_aTextures[ TEXTURE_SPLATTER_5 ] );

         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7dumb01.pcx",  &s_aTextures[ TEXTURE_DEFENSE ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf01.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf02.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_1 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\Umbrella\\7escuf03.pcx",  &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_2 ] );

         InitObject( FX_ID_CURSOR, &g_fxCursor, &s_aTextures[ TEXTURE_CURSOR_0 ],
      	            nMouseX - ( s_aTextures[ TEXTURE_CURSOR_0 ].width >> 1 ),
      	            nMouseY - ( s_aTextures[ TEXTURE_CURSOR_0 ].height >> 1 ) );

         InitObject( FX_ID_DEFENSE, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ],
                     ( nScreenWidth >> 1 ), ( nScreenHeight >> 1 ) );
         g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset state flag
         g_defense.fxObject.dwFlags = 0;                       // Clear erase flag

         // Init table of GST objects
         memset( s_aGST, 0, sizeof( gst_t ) * MAX_OBJECTS_GST );

         // Init table of splatter objects
         memset( s_aSplatter, 0, sizeof( splatter_t ) * MAX_OBJECTS_SPLATTER );
         break;
      }

      case EFFECT_ENDED:
      {
         gst_t *pGST, *pGSTEnd;
         int   i;


         DeleteWaves();       // Free sounds

         for ( i = 0; i < NUM_TEXTURES_TOTAL; i++ )
            DeleteTexture( &s_aTextures[ i ] );

         // Delete all objects that are in the middle of an animation sequence.
         // They don't count as hits on the page.
         pGST     = &s_aGST[ 0 ];
         pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
         while ( pGST < pGSTEnd )
         {
            if ( pGST->dwState )
            {
               pGST->dwState  = GST_STATE_INVALID;
               pGST->pfxGST   = NULL;
               DeleteLastFXObject();      // Remove record of this object from the global
                                          // Projectile[] array so it doesn't get saved.
                                          // This takes care of exiting in the middle of an animation sequence
            }

            pGST++;
         }
         break;
      }

      default:
         break;      // Future commands do not generate an error
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     GST_Draw
*
*  SYNOPSIS
*     iSuccess = GST_Draw( pDDrawData, iMsg );
*
*  DESCRIPTION
*     Draws the current frame for the GST effect
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData     = Info on current DirectDraw device
*     int            iMsg           = Message to process
*
*  OUTPUTS
*		int iSuccess = TRUE
*
*  BUGS
*		None
*
*  SEE ALSO
*     InitEffects()
*
********************************************************************************/
BOOL GST_Draw( DDRAW_DATA *pDDrawData, int iMsg )
{
   GST_CheckInputDevices();

   // Grab defense status from main app variable
   g_defense.byObjectAlive = nEffectsDefended;

   switch( iMsg )
   {
      case DRAW_EFFECT:
         GST_Calculate();
         GST_DrawBuffer( pDDrawData );      // Main drawing pipeline
         GST_DrawScreen( pDDrawData );
         GST_EraseBuffer( pDDrawData );
         break;

      case CALCULATE_AND_SKIP_FRAME:
         GST_Calculate();
         break;

      default:
         break;   // Simply ignore future messages
   }

   return ( TRUE );
}


/******************************************************************************
*
*  NAME
*     GST_CheckInputDevices
*
*  SYNOPSIS
*     GST_CheckInputDevices()
*
*  DESCRIPTION
*     Checks for input from the mouse or keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     GST_Draw(), GST_CheckCheatCodes()
*
********************************************************************************/
static void GST_CheckInputDevices( void )
{
   if ( LButtonPressed )
   {
      LButtonPressed = FALSE;
      PlayWave( s_aiSounds[ SOUND_GRUNT ] ); // Grunt whether or not we have ammo
      GST_Start();                           // Toss a new GST
   }

   // Ignore all other mouse button events
   if ( LButtonReleased )
      LButtonReleased = FALSE;
   if ( RButtonPressed )
      RButtonPressed = FALSE;
   if ( RButtonReleased )
      RButtonReleased = FALSE;

   // Check for cheat codes
   GST_CheckCheatCodes();
}


/******************************************************************************
*
*  NAME
*     GST_CheckCheatCodes
*
*  SYNOPSIS
*     GST_CheckCheatCodes()
*
*  DESCRIPTION
*     Checks for cheat codes typed in from the keyboard.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     GST_CheckInputDevices()
*
********************************************************************************/
static void GST_CheckCheatCodes( void )
{   
   if ( ( szCode[ 0 ] == 'd' ) && !strcmp( szCode, "defense" ) )
   {
      nEffectsDefended ^= TRUE;
      if ( !nEffectsDefended )
         g_defense.fxObject.dwFlags = ERASE;
      ResetCode();
   }
}


/******************************************************************************
*
*  NAME
*     GST_Start
*
*  SYNOPSIS
*     GST_Start()
*
*  DESCRIPTION
*     Tosses a new GST aimed at the current cursor position.
*     Calculates the GST's initial velocity in order to hit the target.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*		Assumes there are 5 frames of animation for the GST throwing sequence.
*
*  SEE ALSO
*     GST_CheckInputDevices()
*
********************************************************************************/
static void GST_Start( void )
{      
   gst_t    *pGST, *pGSTEnd;
   DWORD    dwGSTCount;
   WORD     wFrameDelay;


   // Find room in object array to toss another GST
   dwGSTCount  = 0;
   pGST        = &s_aGST[ 0 ];
   pGSTEnd     = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      if ( pGST->dwState )
      {
         dwGSTCount++;
         if ( dwGSTCount == MAX_OBJECTS_GST )
            return;              // Can't toss any more GSTs for now
      }
      else
      {
         break;                  // Found a free spot, toss a new GST
      }

      pGST++;
   }

   // Start the GST throwing sequence
   memset( pGST, 0, sizeof( *pGST ) );

   pGST->pfxGST = CreateFXObject();
   if ( !( pGST->pfxGST ) )
   {
      // Too many objects on screen already
      return;
   }

   pGST->dwState = GST_STATE_WAITING;

   // The higher the target, the higher the frame delay and hence the longer
   // it takes for the GST to reach its target.
   wFrameDelay = s_awFrameDelays[ ( nMouseY * NUM_OF_FRAME_DELAYS ) / nScreenHeight ];
   pGST->wFrameDelay = wFrameDelay;

   InitObject( FindEffect( EFFECT_NAME_GST ), pGST->pfxGST,
               &s_aTextures[ TEXTURE_GST_0 ],
               nMouseX - ( s_aTextures[ TEXTURE_GST_0 ].width >> 1 ),
               GST_STARTING_Y );
   pGST->pfxGST->nFrame    = TEXTURE_GST_0;
   pGST->pfxGST->dwFlags   = ( DWORD ) nMouseX; // Target's center x position
   pGST->pfxGST->dwFlags2  = ( DWORD ) nMouseY; // Target's center y position

   // Calculate the initial y-velocity of the GST, positive y points upward
   // d = vt + a( t^2 )/2
   // v = ( d - a( t^2)/2 ) / t
   // d = ( GST_STARTING_Y - ( nMouseY - s_aTextures[ TEXTURE_GST_0 ].height / 2 ) )
   // t = time to display all 5 frames of animation = 5 * wFrameDelay
   pGST->pfxGST->nFlags = 
      ( ( GST_STARTING_Y - ( nMouseY - ( s_aTextures[ TEXTURE_GST_0 ].height >> 1  ) ) )
        + ( G_ACCELERATION * ( 14 * wFrameDelay * wFrameDelay ) ) ) / ( 5 * wFrameDelay );

   // Record the starting time so we can wait for the grunt sound to finish
   // before we play the fart sound and actually launch the GST.
   pGST->dwStartTime = GetTickCount(); 
}


/******************************************************************************
*
*  NAME
*     GST_CreateSplatter
*
*  SYNOPSIS
*     GST_CreateSplatter( x, y )
*
*  DESCRIPTION
*     Creates a new splatter object at the given screen coordinates.
*     The velocity and direction are assigned at random.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are 6 different splatter object textures.
*
*  SEE ALSO
*     GST_Calculate()
*
********************************************************************************/
static void GST_CreateSplatter( int x, int y )
{
   splatter_t *pSplatter, *pSplatterEnd;


   pSplatter      = &s_aSplatter[ 0 ];
   pSplatterEnd   = &s_aSplatter[ MAX_OBJECTS_SPLATTER ];
   while ( pSplatter < pSplatterEnd )
   {
      if ( !( pSplatter->dwState ) )
         break;

      pSplatter++;
   }

   if ( pSplatter == pSplatterEnd )
      return;                          // No more room to create another splatter object

   memset( pSplatter, 0, sizeof( splatter_t ) ); // No, this clears out the previous dirty rectangle

   pSplatter->dwState            = SPLATTER_STATE_ALIVE;
   pSplatter->fxObject.nFrame    = TEXTURE_SPLATTER_0 + ( rand() % 6 ); // Assumes there are 6 splatter object textures
   pSplatter->fxObject.x         = x - ( s_aTextures[ TEXTURE_SPLATTER_0 ].width >> 1 );
   pSplatter->fxObject.y         = y - ( s_aTextures[ TEXTURE_SPLATTER_0 ].height >> 1 );

   // Choose a random starting x- and y-velocity.
   pSplatter->fxObject.dwFlags   = ( rand() % ( nScreenWidth >> 5 ) ) - ( nScreenWidth >> 6 );  // x-velocity
   pSplatter->fxObject.dwFlags2  = -( ( rand() % ( nScreenHeight >> 5 ) ) + 5 );                // y-velocity, biased towards upward direction

   // Angle will be calculated in GST_Calculate() before we draw

}


/******************************************************************************
*
*  NAME
*     GST_Calculate
*
*  SYNOPSIS
*     GST_Calculate()
*
*  DESCRIPTION
*     Calculates the position and appropriate frame to display for all
*     objects on the screen.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are 3 different final textures.
*		Assumes there are 3 different impact sounds.
*
*  SEE ALSO
*     GST_Draw(), GST_DefenseMove(), GST_DefenseCheckForCollision()
*
********************************************************************************/
static void GST_Calculate( void )
{
   gst_t       *pGST, *pGSTEnd;
   FXOBJECT    *pfx;
   splatter_t  *pSplatter, *pSplatterEnd;


   // ----- Update the cursor animation and position
   g_fxCursor.nFrameTime++;
   if ( !( g_fxCursor.nFrameTime % DEFAULT_ANIM_FRAME_DELAY ) )
   {
      g_fxCursor.nFrame++;
      if ( g_fxCursor.nFrame > TEXTURE_CURSOR_3 )
         g_fxCursor.nFrame = TEXTURE_CURSOR_0;
   }
   g_fxCursor.x = nMouseX - ( s_aTextures[ TEXTURE_CURSOR_0 ].width >> 1 );
   g_fxCursor.y = nMouseY - ( s_aTextures[ TEXTURE_CURSOR_0 ].height >> 1 );

   // ----- Update GST objects
   pGST     = &s_aGST[ 0 ];
   pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      pfx = pGST->pfxGST;
      switch( pGST->dwState )
      {
         case GST_STATE_INVALID:
            break;

         case GST_STATE_WAITING:
            g_fxCursor.nFrame = TEXTURE_CURSOR_0;  // Stop animating cursor while GST is thrown

            // Wait for the grunt sound to finish before launching the GST with a fart
            if ( ( GetTickCount() - pGST->dwStartTime ) < GST_LAUNCH_DELAY )
               break;

            PlayWave( s_aiSounds[ SOUND_FART_0 + ( rand() & 1 ) ] );
            pGST->dwState = GST_STATE_THROWN;
            break;

         case GST_STATE_THROWN:
            g_fxCursor.nFrame = TEXTURE_CURSOR_0;  // Stop animating cursor while GST is thrown

            pfx->nFrameTime++;
            if ( pfx->nFrame < TEXTURE_FINAL_0 )
            {
               // Update velocity and position of GST as it travels through the air
               pfx->nFlags   -= G_ACCELERATION;   // Positive velocity points upward
               pfx->y        -= pfx->nFlags;
            }

            if ( !( pfx->nFrameTime % pGST->wFrameDelay ) )
            {
               pfx->nFrame++;
               if ( pfx->nFrame == TEXTURE_FINAL_0 )
               {
                  int   iNumSplatterObjects;


                  // Recenter image over original target
                  pfx->x = pfx->dwFlags - ( s_aTextures[ TEXTURE_FINAL_0 ].width >> 1 );
                  pfx->y = pfx->dwFlags2 - ( s_aTextures[ TEXTURE_FINAL_0 ].height >> 1 );
                  pfx->nFrame += rand() % 3;    // Assumes 3 different final textures
                  pGST->dwState = GST_STATE_HIT_PAGE;
                  PlayWave( s_aiSounds[ SOUND_HIT_PAGE_0 + ( rand() % 3 ) ] );   // !!! Assumes 3 different plop sounds

                  iNumSplatterObjects = ( rand() % ( MAX_SPLATTER_PER_GST - MIN_SPLATTER_PER_GST + 1 ) ) + MIN_SPLATTER_PER_GST;
                  while ( iNumSplatterObjects > 0 )
                  {
                     GST_CreateSplatter( pfx->x + ( s_aTextures[ TEXTURE_FINAL_0 ].width >> 1 ), pfx->y + ( s_aTextures[ TEXTURE_FINAL_0 ].height >> 1 ) );
                     iNumSplatterObjects--;
                  }
               }
            }
            break;

         case GST_STATE_HIT_PAGE:
            break;

         case GST_STATE_BLOCKED:
               // Update velocity and position of GST as it travels through the air
            pfx->nFlags   -= G_ACCELERATION;   // Positive velocity points upward
            pfx->y        -= pfx->nFlags;
            break;

         default:
            break;
      }

      pGST++;
   }

   // ----- Update splatter objects
   pSplatter      = &s_aSplatter[ 0 ];
   pSplatterEnd   = &s_aSplatter[ MAX_OBJECTS_SPLATTER ];
   while ( pSplatter < pSplatterEnd )
   {
      if ( pSplatter->dwState == SPLATTER_STATE_ALIVE )
      {
         pfx = &( pSplatter->fxObject );

         // Kill splatter objects that are offscreen
         if ( ( pfx->x < -( s_aTextures[ TEXTURE_SPLATTER_0 ].width ) ) ||
              ( pfx->x > nScreenWidth ) ||
              ( pfx->y > nScreenHeight ) )
         {
            pSplatter->dwState = SPLATTER_STATE_ERASE;
            goto next;
         }

         pfx->x += ( int )( pfx->dwFlags );
         pfx->y += ( int )( pfx->dwFlags2 );

         // Adjust y-velocity for gravity
         pfx->dwFlags2 += G_ACCELERATION;

         // Angle depends on current velocity, angle of 0 points to the right
         // Increasingly positive angles move counterclockwise
         // Convert radians to degrees
         pfx->fAngle = ( float )( ( atan2( -( double )( ( int )( pfx->dwFlags2 ) ), ( double )( ( int )( pfx->dwFlags ) ) ) * 180.0f ) / PI );
      }
next:
      pSplatter++;
   }

   if ( g_defense.byObjectAlive )
   {
      GST_DefenseCheckForCollision();
      GST_DefenseMove();
   }
}


/******************************************************************************
*
*  NAME
*     GST_DrawBuffer
*
*  SYNOPSIS
*     GST_DrawBuffer( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects into the offscreen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     GST_Draw()
*
********************************************************************************/
static void GST_DrawBuffer( DDRAW_DATA *pDDrawData )
{
   gst_t       *pGST, *pGSTEnd;
   FXOBJECT    *pfx;
   splatter_t  *pSplatter, *pSplatterEnd;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawObjectToBuffer( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ], 0 );

   // Draw GST objects
   pGST     = &s_aGST[ 0 ];
   pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      pfx = pGST->pfxGST;
      switch( pGST->dwState )
      {
         case GST_STATE_INVALID:
         case GST_STATE_WAITING:
            break;

         case GST_STATE_THROWN:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         case GST_STATE_HIT_PAGE:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            DrawObjectToScreenCapture( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         case GST_STATE_BLOCKED:
            DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], 0 );
            break;

         default:
            break;
      }

      pGST++;
   }

   // Draw splatter objects
   pSplatter      = &s_aSplatter[ 0 ];
   pSplatterEnd   = &s_aSplatter[ MAX_OBJECTS_SPLATTER ];
   while ( pSplatter < pSplatterEnd )
   {
      pfx = &( pSplatter->fxObject );
      if ( pSplatter->dwState == SPLATTER_STATE_ALIVE )
         DrawObjectToBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ], ROTATE_IMAGE );

      pSplatter++;
   }

   // Draw cursor
   DrawObjectToBuffer( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ], 0 );
}


/******************************************************************************
*
*  NAME
*     GST_DrawScreen
*
*  SYNOPSIS
*     GST_DrawScreen( pDDrawData )
*
*  DESCRIPTION
*     Draws all objects from the offscreen buffer onto the front screen buffer.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     GST_Draw()
*
********************************************************************************/
static void GST_DrawScreen( DDRAW_DATA *pDDrawData )
{
   gst_t       *pGST, *pGSTEnd;
   FXOBJECT    *pfx;
   splatter_t  *pSplatter, *pSplatterEnd;


   // Draw defense
   if ( g_defense.byObjectAlive )
      DrawBufferObjectToScreen( pDDrawData, &g_defense.fxObject, &s_aTextures[ TEXTURE_DEFENSE ] );

   // Draw GST objects
   pGST     = &s_aGST[ 0 ];
   pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      pfx = pGST->pfxGST;
      switch( pGST->dwState )
      {
         case GST_STATE_INVALID:
         case GST_STATE_WAITING:
            break;

         case GST_STATE_THROWN:
         case GST_STATE_HIT_PAGE:
         case GST_STATE_BLOCKED:
            DrawBufferObjectToScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            break;

         default:
            break;
      }

      pGST++;
   }

   // Draw splatter objects
   pSplatter      = &s_aSplatter[ 0 ];
   pSplatterEnd   = &s_aSplatter[ MAX_OBJECTS_SPLATTER ];
   while ( pSplatter < pSplatterEnd )
   {
      pfx = &( pSplatter->fxObject );
      if ( pSplatter->dwState == SPLATTER_STATE_ALIVE )
         DrawBufferObjectToScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );

      pSplatter++;
   }

   // Draw cursor
   DrawBufferObjectToScreen( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ] );

   // Erase the defense
   if ( g_defense.fxObject.dwFlags == ERASE )
   {
      EraseObjectInBuffer( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );
      EraseObjectOnScreen( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );
      g_defense.fxObject.dwFlags = 0;
      g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Reset the defense mechanism
   }
}


/******************************************************************************
*
*  NAME
*     GST_EraseBuffer
*
*  SYNOPSIS
*     GST_EraseBuffer( pDDrawData )
*
*  DESCRIPTION
*     Erases all the objects that were previously drawn into the offscreen
*     buffer in order to restore the offscreen buffer back to its original state
*     so we can draw on a clean background again on the next iteration of
*     the drawing pipeline.
*
*     This also handles the ending state transitions, except for the defense mechanism.
*
*  INPUTS
*     DDRAW_DATA *pDDrawData = Current DirectDraw device info.
*
*  OUTPUTS
*     None
*
*  BUGS
*		None
*
*  SEE ALSO
*     GST_Draw()
*
********************************************************************************/
static void GST_EraseBuffer( DDRAW_DATA *pDDrawData )
{
   gst_t       *pGST, *pGSTEnd;
   FXOBJECT    *pfx;
   splatter_t  *pSplatter, *pSplatterEnd;


   // Erase GST objects
   pGST     = &s_aGST[ 0 ];
   pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      pfx = pGST->pfxGST;
      switch( pGST->dwState )
      {
         case GST_STATE_INVALID:
         case GST_STATE_WAITING:
            break;

         case GST_STATE_THROWN:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            break;

         case GST_STATE_HIT_PAGE:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            pGST->dwState  = GST_STATE_INVALID;
            pfx->nFlags    = 0;     // Clear drawing flags for automatically saved object
            pGST->pfxGST   = NULL;  // Note that the global Projectile[] array will still contain the
                                    // exploded hole from this GST, which is what we want
            break;

         case GST_STATE_BLOCKED:
            EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
            if ( pfx->y > nScreenHeight )
            {
//               EraseObjectOnScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );

               // Blocked GST has fallen off the screen, we can safely remove it now
               pfx->x = -9999;   // Kluge, we move the object out of bounds so it won't be drawn.
               pfx->y = -9999;   // The entry in the global Projectiles[] array will be erased later
                                 // when the effect is ended and the array is condensed
               pGST->dwState  = GST_STATE_INVALID;
               pGST->pfxGST   = NULL;
            }
            break;

         default:
            break;
      }

      pGST++;
   }

   // Erase splatter objects
   pSplatter      = &s_aSplatter[ 0 ];
   pSplatterEnd   = &s_aSplatter[ MAX_OBJECTS_SPLATTER ];
   while ( pSplatter < pSplatterEnd )
   {
      pfx = &( pSplatter->fxObject );
      if ( pSplatter->dwState == SPLATTER_STATE_ALIVE )
      {
         EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
      }
      else if ( pSplatter->dwState == SPLATTER_STATE_ERASE )
      {
         EraseObjectInBuffer( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
         EraseObjectOnScreen( pDDrawData, pfx, &s_aTextures[ pfx->nFrame ] );
         pSplatter->dwState = SPLATTER_STATE_INVALID;
      }
      pSplatter++;
   }

   // Erase defense
   if ( g_defense.byObjectAlive )
      EraseObjectInBuffer( pDDrawData, &( g_defense.fxObject ), &s_aTextures[ TEXTURE_DEFENSE ] );

   // Erase cursor
   EraseObjectInBuffer( pDDrawData, &g_fxCursor, &s_aTextures[ g_fxCursor.nFrame ] );
}


/******************************************************************************
*
*  NAME
*		GST_DefenseMove
*
*  SYNOPSIS
*     GST_DefenseMove()
*
*  DESCRIPTION
*     Determines where the umbrella should move for the time being.
*     The umbrella will seek the first GST scheduled to hit the page,
*     or it simply follows the cursor if there are no GSTs being thrown.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     None
*
*  SEE ALSO
*     GST_Calculate(), GST_DefenseCheckForCollision()
*
********************************************************************************/
static void GST_DefenseMove( void )
{
   gst_t    *pGST, *pGSTEnd;
   FXOBJECT *pfx;


   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_SEEKING )
   {
      int         dx, dy;
      int         nFrameTarget;


      // Find a new target, which can be either the mouse cursor, 
      // or the first GST scheduled to hit the page.
      // The first GST to hit the page is simply the one with the
      // highest nFrameTime member, which indicates how long it's been alive.
      g_defense.nDestX  = DEFENSE_FOLLOW_CURSOR;   // Assume no GSTs on screen for now
      nFrameTarget      = -1;
      pGST              = &s_aGST[ 0 ];
      pGSTEnd           = &s_aGST[ MAX_OBJECTS_GST ];
      while ( pGST < pGSTEnd )
      {
         pfx = pGST->pfxGST;
         if ( pGST->dwState == GST_STATE_THROWN )
         {
            if ( pfx->nFrameTime > nFrameTarget )
            {
               nFrameTarget      = pfx->nFrameTime;
               g_defense.nDestX  = ( int )( pfx->dwFlags );
               g_defense.nDestY  = ( int )( pfx->dwFlags2 );
            }
         }

         pGST++;
      }

      if ( g_defense.nDestX == DEFENSE_FOLLOW_CURSOR )
      {
         g_defense.nDestX = nMouseX;
         g_defense.nDestY = nMouseY;
      }

      g_defense.nSpeed = ( rand() % ( DEFENSE_MAX_STEP - DEFENSE_MIN_STEP + 1 ) ) + DEFENSE_MIN_STEP;

      // Umbrella does not span the entire surface of the bitmap
      // So we use hardcoded values for the actual width and height
      dx = g_defense.nDestX - ( g_defense.fxObject.x + DEFENSE_X1 + ( DEFENSE_X2 >> 1 ) );
      dy = g_defense.nDestY - ( g_defense.fxObject.y + DEFENSE_Y1 + ( DEFENSE_Y2 >> 1 ) );

      g_defense.fIncrX = ( float ) dx / ( float ) g_defense.nSpeed;
      g_defense.fIncrY = ( float ) dy / ( float ) g_defense.nSpeed;

      g_defense.nCount = 0;      // This keeps track of which step we're on
      g_defense.fxObject.nFlags = DEFENSE_STATE_MOVING;
   }

   // ----- Move the defense if applicable
   if ( g_defense.fxObject.nFlags == DEFENSE_STATE_MOVING )
   {
      g_defense.fxObject.x += ( int ) g_defense.fIncrX;
      g_defense.fxObject.y += ( int ) g_defense.fIncrY;

      g_defense.nCount++;                          // Increment the current step counter
      if ( g_defense.nCount > g_defense.nSpeed )   // We've reached our target
         g_defense.fxObject.nFlags = DEFENSE_STATE_SEEKING;
   }
}


/******************************************************************************
*
*  NAME
*		GST_DefenseCheckForCollision
*
*  SYNOPSIS
*     GST_DefenseCheckForCollision()
*
*  DESCRIPTION
*     Checks to see if the umbrella has blocked a GST.
*
*  INPUTS
*     None
*
*  OUTPUTS
*     None
*
*  BUGS
*     Assumes there are three different damage textures.
*
*  SEE ALSO
*     GST_Calculate(), GST_DefenseMove()
*
********************************************************************************/
static void GST_DefenseCheckForCollision( void )
{
   gst_t    *pGST, *pGSTEnd;
   TEXTURE  *pTextureUmbrella, *pTextureDamage;


   pTextureUmbrella  = &s_aTextures[ TEXTURE_DEFENSE ];
   pTextureDamage    = &s_aTextures[ TEXTURE_DEFENSE_DAMAGE_0 + ( rand() % 3 ) ];   // Assumes 3 different damage textures

   // ----- Determine if any objects have just hit the umbrella
   pGST     = &s_aGST[ 0 ];
   pGSTEnd  = &s_aGST[ MAX_OBJECTS_GST ];
   while ( pGST < pGSTEnd )
   {
      if ( pGST->dwState == GST_STATE_HIT_PAGE )
      {
         int         iDefenseX1, iDefenseX2, iDefenseY1, iDefenseY2;
         int         iWeaponX1, iWeaponX2, iWeaponY1, iWeaponY2;
         FXOBJECT *  pfxObject;


         pfxObject = pGST->pfxGST;

         // Check for collision using bounding boxes.
         iDefenseX1 = g_defense.fxObject.x + DEFENSE_X1;
         iDefenseX2 = g_defense.fxObject.x + DEFENSE_X2;
         iDefenseY1 = g_defense.fxObject.y + DEFENSE_Y1;
         iDefenseY2 = g_defense.fxObject.y + DEFENSE_Y2;

         iWeaponX1 = pfxObject->x + GST_COLLISION_BOX_X1;
         iWeaponX2 = pfxObject->x + GST_COLLISION_BOX_X2;
         iWeaponY1 = pfxObject->y + GST_COLLISION_BOX_Y1;
         iWeaponY2 = pfxObject->y + GST_COLLISION_BOX_Y2;

         if ( ( iDefenseX1 >= iWeaponX2 ) || ( iDefenseX2 <= iWeaponX1 ) ||
              ( iDefenseY1 >= iWeaponY2 ) || ( iDefenseY2 <= iWeaponY1 ) )
            goto next;        // Trivial reject, rectangles do not intersect at all

    	   // Weapon collides with the defense!
         // Center damage image at target x and y relative to umbrella
         g_defense.ptDef.x = ( int )( pfxObject->dwFlags ) - ( pTextureDamage->width >> 1 ) - g_defense.fxObject.x;
         g_defense.ptDef.y = ( int )( pfxObject->dwFlags2 ) - ( pTextureDamage->height >> 1 ) - g_defense.fxObject.y;

         if ( ( g_defense.ptDef.x >= DEFENSE_DRAW_X1 ) &&
              ( ( g_defense.ptDef.x + pTextureDamage->width ) <= DEFENSE_DRAW_X2 ) &&
              ( g_defense.ptDef.y >= DEFENSE_DRAW_Y1 ) &&
              ( ( g_defense.ptDef.y + pTextureDamage->height ) <= DEFENSE_DRAW_Y2 ) )
         {
            DrawTextureInTexture( pTextureUmbrella, pTextureDamage, &g_defense.ptDef );
         }

         pGST->dwState              = GST_STATE_BLOCKED;
         pGST->pfxGST->nFlags       = 12;                      // Reset y-velocity to zero
         g_defense.fxObject.nFlags  = DEFENSE_STATE_SEEKING;   // Seek new target
         PlayWave( s_aiSounds[ SOUND_BLOCKED ] );
      }
next:
      pGST++;
   }

}


/******************************************************************************
*
*  NAME
*		GST_ManualSave
*
*  SYNOPSIS
*     iSuccess = GST_ManualSave( pDDrawData, iMsg, pvData )
*
*  DESCRIPTION
*     Takes care of displaying the correct final image for the GST effect.
*     Also loads and saves effect data in the same manner as the manual save
*     functions for the other weapon packs.
*
*  INPUTS
*     DDRAW_DATA *   pDDrawData  = Pointer to DirectDraw device.
*     int            iMsg        = Message to process, i.e. load, save, or display.
*     void *         pvData      = Data location for reads or writes.
*
*  OUTPUTS
*     int iSuccess = TRUE
*
*  BUGS
*     Assumes 3 different final textures, which is hardcoded instead of using a macro.
*
*  SEE ALSO
*
********************************************************************************/
int GST_ManualSave( DDRAW_DATA *pDDrawData, int iMsg, void *pvData )
{
   static   BOOL              s_bTexturesLoaded = FALSE;
   static   TEXTURE           s_aSaveTextures[ 3 ] = { { 0 }, { 0 }, { 0 } };
            TEXTURE *         pTexture;
            FXOBJECT *        pfxObject;
            URLDATA *         pURLData;
            gstSaveData_t     *pSaveData, *pSaveDataEnd;
            int               iEffectID;


   iEffectID = FindEffect( EFFECT_NAME_GST );

   switch( iMsg )
	{
	case MANUAL_DRAW_TO_BROWSER:
      if ( !s_bTexturesLoaded )
      {
         // Make sure these texture file names match with the LoadTexture() calls above
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturda.pcx",   &s_aSaveTextures[ TEXTURE_FINAL_0 - TEXTURE_FINAL_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturdb.pcx",   &s_aSaveTextures[ TEXTURE_FINAL_1 - TEXTURE_FINAL_0 ] );
         LoadTexture( pDDrawData, ".\\Images\\Poo\\GST\\7oturdc.pcx",   &s_aSaveTextures[ TEXTURE_FINAL_2 - TEXTURE_FINAL_0 ] );
         s_bTexturesLoaded = TRUE;
      }

		pfxObject   = ( FXOBJECT * ) pvData;
      if ( ( pfxObject->nFrame < TEXTURE_FINAL_0 ) || ( pfxObject->nFrame > TEXTURE_FINAL_2 ) )
         break;

      pTexture = &s_aSaveTextures[ pfxObject->nFrame - TEXTURE_FINAL_0 ];
      DrawObjectToBrowserBuffer( pDDrawData, pfxObject, pTexture, 0 );
		break;

   case MANUAL_LOADED_EFFECT_DATA:
      pURLData       = ( URLDATA * ) pvData;
      pSaveData      = ( gstSaveData_t * )( pURLData->ptrData );
		pSaveDataEnd   = ( gstSaveData_t * )( ( DWORD ) pSaveData + ( DWORD ) ( pURLData->nSize ) );

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = CreateFXObject();
         if ( !pfxObject )
            break;

         InitObject( iEffectID, pfxObject, NULL, 0, 0 );
         pfxObject->x               = pSaveData->x;
         pfxObject->y               = pSaveData->y;
         pfxObject->nFrame          = pSaveData->iFrame;
         pfxObject->nInsideBrowser  = TRUE;

         pSaveData++;
      }

      free( pURLData->ptrData ); // This is not very kosher!
      break;

	case MANUAL_SAVE_EFFECT_DATA:
      pURLData          = ( URLDATA * ) pvData;
      pURLData->nSize   = GetTotalFXObjectsForEffect( iEffectID ) * sizeof( gstSaveData_t );
      pURLData->ptrData = malloc( pURLData->nSize );
      pSaveData         = ( gstSaveData_t * )( pURLData->ptrData );
      pSaveDataEnd      = ( gstSaveData_t * )( ( DWORD ) pSaveData + ( DWORD )( pURLData->nSize ) );
      if ( !pSaveData )
         break;                                 // Error! malloc() failed!

      pfxObject = GetNextFXObject( iEffectID, FIND_FIRST_OBJECT );
      if ( !pfxObject )
         break;

      pSaveData->x      = pfxObject->x;
      pSaveData->y      = pfxObject->y;
      pSaveData->iFrame = pfxObject->nFrame;
      pSaveData++;

      while ( pSaveData < pSaveDataEnd )
      {
         pfxObject = GetNextFXObject( iEffectID, FIND_NEXT_OBJECT );
         if ( !pfxObject )
            break;

     		pSaveData->x      = pfxObject->x;
     		pSaveData->y      = pfxObject->y;
     		pSaveData->iFrame = pfxObject->nFrame;

         pSaveData++;
    	}
    	break;
	}

   return ( TRUE );
}



